'''
Computes statistics for SATzilla
'''

import csv
import sys
import os
import json

BACKUP_SOLVER = 'g12cpx'
FEAT_SOLVER = 'gecode'
TIMEOUT = 1800.0
NUM_SOLVER = 12
FEAT_PATH = '/public/xcsp_instances/new/scripts/classifiers/cop/SATzilla/features/'
PRED_PATH = '/public/xcsp_instances/new/predictions/cop/SATzilla/'
TEST_PATH = '/public/xcsp_instances/new/scripts/classifiers/cop/SATzilla/test/'
INFOS = '/public/xcsp_instances/new/scripts/solvers/cop/infos'

def main(args):
  with open(INFOS, 'r') as infile:
    infos = json.load(infile)
  stats_writer = csv.writer(open(PRED_PATH + 'stats_SATzilla', 'wb'),  delimiter = '|')
  for n in range(2, NUM_SOLVER + 1):
    counter = 0
    asp = 0.0
    pof = 0.0
    psi = 0.0
    aot = 0.0
    ast = 0.0
    pred_writer = csv.writer(open(PRED_PATH + str(n) + '_solvers/pred_' + str(n) + 'solvers', 'wb'),  delimiter = '|')
    for rep in range(1, 6):
      for fold in range(1, 6):
        suffix = 'rep' + str(rep) + '_fold' + str(fold)
        feat = FEAT_PATH + 'test_' + suffix + '.features'
        test = TEST_PATH + str(n) + '_solvers/test_' + str(n) + 'solvers_' + suffix
        feat_reader = csv.reader(open(feat, 'rb'),  delimiter = ' ')
        test_reader = csv.reader(open(test, 'rb'),  delimiter = ' ')
        for row in feat_reader:
          counter += 1
          instance = row[0]
          solver = test_reader.next()[0]
          item = infos[instance][solver]
          score = item['score']
          time  = item['time']
          if solver == FEAT_SOLVER:
            time += item['feat_time'] - item['fzn_time']
          else:
            time += item['feat_time']
	  asp += score
	  if score > 0:
	    psi += 1
	    if score == 1:
	      aot += time
	      pof += 1
	    else:
	      aot += TIMEOUT
	    ast += time
	  else:
	    ast += TIMEOUT
            aot += TIMEOUT
          pred_writer.writerow([instance, rep, fold, asp, pof, psi, aot, ast, solver])
    asp = str(100 * asp / counter)
    pof = str(100 * pof / counter)
    psi = str(100 * psi / counter)
    aot = str(aot / counter)
    ast = str(ast / counter) 
    stats_writer.writerow([n, asp, pof, psi, aot, ast])

if __name__ == '__main__':
    main(sys.argv[1:])
