'''
Compute statistics for off-the-shelf classifiers (called by compute_statistics_ots).
'''
import sys
import os
import json

BACKUP_SOLVER = 'g12cpx'
FEAT_SOLVER = 'gecode'
TIMEOUT = 1800.0
infos_file = '/public/xcsp_instances/new/scripts/solvers/cop/infos'

def compute_predictions(pred_path, infos):
  asp = 0.0
  pof = 0.0
  psi = 0.0
  aot = 0.0
  ast = 0.0
  counter = 0
  dir_list = sorted(os.listdir(pred_path))
    
  for file_name in dir_list:
    f = open(pred_path + "/" + file_name)
    # Skip the first line.
    f.readline() 
    line = f.readline()
    while line:
      row = line.split()
      if len(row) > 0:
        counter += 1
        # Get the predicted solver and the instance name.
        solver = row[2].split(':')[1]
        instance = row[-1][1:-1]
        if solver == 'minisati' or solver == 'minisat':
          solver = 'minisatid'
        elif solver == 'g12lazyf' or solver == 'g12lazy':
          solver = 'g12lazyfd'
        elif solver == 'nosolver' or solver == 'nosolve':
          solver = BACKUP_SOLVER
        item = infos[instance][solver]
        score = item['score']
        time  = item['time']
        if solver == FEAT_SOLVER:
          time += item['feat_time'] - item['fzn_time']
        else:
          time += item['feat_time']
	asp += score
	if score > 0:
	  psi += 1
	  if score == 1:
	    aot += time
	    pof += 1
	  else:
	    aot += TIMEOUT
	  ast += time
	else:
	  ast += TIMEOUT
          aot += TIMEOUT
      line = f.readline()
  asp = str(100 * float(asp) / counter)
  pof = str(100 * float(pof) / counter)
  psi = str(100 * float(psi) / counter)
  aot = str(aot / counter)
  ast = str(ast / counter)
  return asp + '|' + pof + '|' + psi + '|' + aot + '|' + ast
        
def main(args):
  with open(infos_file, 'r') as infile:
    infos = json.load(infile)
  cp = compute_predictions(args[0], infos)
  print cp

if __name__ == '__main__':
    main(sys.argv[1:])
