'''
Compute the dataset for each portfolio, associating for each instance the 
corresponding label.
'''

import csv
import sys
import json

portfolios_file = '/public/xcsp_instances/new/scripts/solvers/cop/portfolios_cop'
features_file = '/public/xcsp_instances/new/features/cop/features_head.csv'
dataset_path = '/public/xcsp_instances/new/dataset/cop/csv/'
infos_file = '/public/xcsp_instances/new/scripts/solvers/cop/infos'

def compute_dataset(num_solvers, portfolio, infos):
  dataset_writer = csv.writer(
    open(dataset_path + 'dataset_' + str(num_solvers) + '_solvers.new', 'wb')
  )
  features_reader = csv.reader(open(features_file, 'rb'),  delimiter = '|')
  header = features_reader.next()
  header.append('label')
  dataset_writer.writerow(header)
  for row in features_reader:
    item = infos[row[0]]
    port_infos = [(
      1 - item[s]['score'], item[s]['time'], item[s]['info'], s
      ) for s in portfolio
    ]
    rank = sorted(port_infos) 
    (score, time, info, solver) = rank[0]
    if score == 1:
      label = 'nosolver'
    else:
      label = solver
    row.append(label)
    dataset_writer.writerow(row)

def main(args):
  portfolios_reader = csv.reader(open(portfolios_file,  'rb'),  delimiter = '|')
  with open(infos_file, 'r') as infile:
    infos = json.load(infile)
  for row in portfolios_reader:
    compute_dataset(row[0], eval(row[1]), infos)  

if __name__ == '__main__':
    main(sys.argv[1:])
