'''
Compute the elections with different criteria (viz. Approval, Borda, Plurality)
'''
import sys
import csv
import json

times_file='/public/xcsp_instances/new/scripts/solvers/cop/infotimevals_cop'
feat_file = '/public/xcsp_instances/new/features/cop/features.csv'
info_file = '/public/xcsp_instances/new/scripts/solvers/cop/infos'
solvers_rank = []
num_solvers = 12
results = {}
feat_vector = {}
infos = {}
voters = 0
abstained = 0
FEAT_INST = 0
FEAT_GOAL = 109 - 1
FEAT_FZN  = 153 - 1
FEAT_TIME = 155 - 1
TIMEOUT = 1800.0

def init_results():
  global results, voters, abstained
  voters = 0
  abstained = 0
  results = {
    'bprolog': 0, 'fzn2smt': 0, 'g12cpx': 0, 'g12fd': 0, 'g12lazyfd': 0, 
    'g12mip': 0, 'gecode': 0, 'izplus': 0, 'jacop': 0, 'minisatid': 0, 
    'mistral': 0, 'ortools': 0
  }
  
def compute_score(info, min_val, max_val, obj, val):
  if info == "unknown":
    return 0
  if info != "sat":
    return 1
  if min_val == max_val:
    return 0.75
  s = ((val - min_val) * 0.5) / (max_val - min_val)
  assert(0 <= s <= 0.5)
  if obj == "min":
    return 0.75 - s
  if obj == "max":
    return s + 0.25
  print "Error!"
  system.exit(1)
    
def fix_objective(inst):
  if 'pattern_set_mining_k1' in inst:
    return 'max'
  elif 'radiation' in inst:
    return 'min'
  else:
    print 'Unknown instance:',inst, int(feat_vector[inst][FEAT_GOAL])
    sys.exit(0)

def approval_rule(inst):
  global voters, abstained, feat_vector
  solvers = infos[inst]
  approval_list = filter(lambda item: item[1]['info'] != 'unknown', solvers.items())
  if int(feat_vector[inst][FEAT_GOAL]) == 2:
    obj = 'min'
  elif int(feat_vector[inst][FEAT_GOAL]) == 3:
    obj = 'max'
  else:
    obj = fix_objective(inst)
  for solver in solvers.keys():
    solvers[solver]['obj'] = obj
    solvers[solver]['fzn_time']  = float(feat_vector[inst][FEAT_FZN])
    solvers[solver]['feat_time'] = float(feat_vector[inst][FEAT_TIME])
    if not approval_list:
      solvers[solver]['score'] = 0
      solvers[solver]['time'] = TIMEOUT   
      solvers[solver]['min_val'] = float('nan')
      solvers[solver]['max_val'] = float('nan')
  if approval_list:
    voters += 1
    min_val = min([solvers[solver]['val'] for (solver, _) in approval_list])
    max_val = max([solvers[solver]['val'] for (solver, _) in approval_list])
    for solv in solvers.keys():
      solvers[solv]['min_val'] = min_val
      solvers[solv]['max_val'] = max_val
      if solv in [s for (s, i) in approval_list]:
        item = solvers[solv]
        score = compute_score(item['info'], min_val, max_val, obj, item['val'])
        results[solv] += score
        item['score']  = score
      else:
        solvers[solv]['score'] = 0
        solvers[solv]['time']  = TIMEOUT
  else:
    abstained += 1
      
def borda_rule(inst):
  global voters, abstained
  solvers = infos[inst]
  approval_list = filter(lambda item: item[1]['info'] != 'unknown', solvers.items())
  if not approval_list:  
    abstained += 1
    return
  voters += 1
  times = []
  for (solv, _) in approval_list:
    score = solvers[solv]['score']
    time  = solvers[solv]['time']
    times.append((1 - score, time, solvers_rank.index(solv), solv))
  rank = sorted(times)
  global num_solvers
  n = num_solvers
  for item in rank:
    results[item[3]] += n
    n -= 1
  
def plurality_rule(inst):
  global voters, abstained
  solvers = infos[inst]
  approval_list = filter(lambda item: item[1]['info'] != 'unknown', solvers.items())
  if not approval_list:  
    abstained += 1
    return
  voters += 1
  times = []
  for (solv, _) in approval_list:
    score = solvers[solv]['score']
    time  = solvers[solv]['time']
    times.append((1 - score, time, solvers_rank.index(solv), solv))
  best = min(times)[3]
  global num_solvers
  results[best] += 1

def main(args):
  feat_reader  = csv.reader(open(feat_file,  'rb'), delimiter = '|')
  feat_vect_file = '/public/xcsp_instances/new/features/cop/feat_vector'
  for row in feat_reader:
    feat_vector[row[0]] = row[1 : len(row)]
  with open(feat_vect_file, 'w') as outfile:
    json.dump(feat_vector, outfile)
  
  times_reader = csv.reader(open(times_file, 'rb'), delimiter = '|')
  init_results()
  global infos
  for row in times_reader:
    inst = row[0]
    infos[inst] = {}
    solvers = eval(row[1])
    for (solver, (info, time, val, vals)) in solvers.items():
      infos[inst][solver] = {}
      infos[inst][solver]['info'] = info
      infos[inst][solver]['time'] = time
      infos[inst][solver]['val']  = float(val)
      infos[inst][solver]['vals'] = vals
    approval_rule(inst)
  approval_res = \
    sorted(results.items(), key = lambda item: item[1], reverse = True)
  print "Approval:", approval_res 
  print "Voters:", voters
  print "Abstained:", abstained
  print "Total:", voters + abstained
  print "----------\n"
  with open(info_file, 'w') as outfile:
    json.dump(infos, outfile)
  global solvers_rank
  solvers_rank = [item[0] for item in approval_res]
  
  init_results()
  for inst in infos.keys():
    borda_rule(inst)
  print "Borda:", \
    sorted(results.items(), key = lambda item: item[1], reverse = True)
  print "Voters:", voters
  print "Abstained:", abstained
  print "Total:", voters + abstained
  print "----------\n"
  
  init_results()
  for inst in infos.keys():
    plurality_rule(inst)
  print "Plurality:", \
    sorted(results.items(), key = lambda item: item[1], reverse = True)
  print "Voters:", voters
  print "Abstained:", abstained
  print "Total:", voters + abstained
  print "----------\n"
  
if __name__ == '__main__':
    main(sys.argv[1:])
