'''
Computes proper information for each solver.
'''
import sys
import csv

res_file  = '/public/xcsp_instances/new/solvers/cop/results/results_cop'
times = {}
infos = {}
error = {}
vals = {}
opts = {}
infotimevals = {}
SOLV = 0
INST = 1
INFO = 4
VAL  = 5
VALS = 6
MZN2FZN = 7
TIME = 8
TIMEOUT = 1800.0
FEAT_SOLVER = 'gecode'

def main(args):
  res_reader          = csv.reader(open(res_file,  'rb'), delimiter = '|')
  times_writer        = csv.writer(open('times_cop',     'wb'), delimiter = '|')
  infos_writer        = csv.writer(open('infos_cop',     'wb'), delimiter = '|')
  error_writer        = csv.writer(open('error_cop',     'wb'), delimiter = '|')
  infotimevals_writer = csv.writer(open('infotimevals_cop', 'wb'), delimiter = '|')
  
  flat_times_writer = csv.writer(open('flat_times_cop.csv', 'wb'), delimiter = '|')
  flat_infos_writer = csv.writer(open('flat_infos_cop.csv', 'wb'), delimiter = '|')
  flat_error_writer = csv.writer(open('flat_error_cop.csv', 'wb'), delimiter = '|')
  
  fzn_times = {}
  for row in res_reader:
    inst   = row[INST]
    info   = row[INFO]
    solv   = row[SOLV]
    val    = row[VAL]
    values = eval(row[VALS])
    time   = float(row[TIME]) + float(row[MZN2FZN])
    if time >= TIMEOUT or info == 'unknown':
      time = TIMEOUT
    infotimevals.setdefault(inst, {})[solv] = (info, time, val, values)

    times.setdefault(inst, {})[solv] = time
    infos.setdefault(inst, {})[solv] = info
    vals.setdefault(inst, {})[solv] = val
    if info == 'opt':
      opts.setdefault(inst, {})[solv] = val
    else:
      opts.setdefault(inst, {})[solv] = 'nan'
  
  header = ['inst']
  header.extend(infos.values()[0].keys())
  flat_infos_writer.writerow(header)
  flat_times_writer.writerow(header)
  flat_error_writer.writerow(header)

  for row in opts.items():
    row_opts = row[1].values()
    if len(set(row_opts)) < 3:
      continue
    inst = row[0]
    line = [inst]
    line.extend(row_opts)
    error[inst] = row[1]
    error_writer.writerow(row)
    flat_error_writer.writerow(line)
  
  for row in infos.items():
    row_infos = row[1].values()
    inst = row[0]
    line = [inst]
    line.extend(row_infos)
    if ('unsat' in row_infos and ('sat' in row_infos or 'opt' in row_infos or 'unbounded' in row_infos)) \
      or ('unbounded' in row_infos and ('opt' in row_infos or 'unsat' in row_infos)):
	error[inst] = row[1]
        error_writer.writerow(row)
        flat_error_writer.writerow(line)
    else:
      infos_writer.writerow(row)
      flat_infos_writer.writerow(line)

  for row in times.items():
    inst = row[0]
    times_writer.writerow(row)
    line = [inst]
    line.extend(row[1].values())
    flat_times_writer.writerow(line)
    
  for row in infotimevals.items():
    infotimevals_writer.writerow(row)  

if __name__ == '__main__':
    main(sys.argv[1:])
