'''
Compute statistics for single solvers and VBS.
'''
import sys
import os
import csv
import json

solvers = [
  'bprolog', 'fzn2smt', 'g12cpx', 'g12fd', 'g12lazyfd', 'g12mip', 'gecode', 
  'izplus', 'jacop', 'minisatid', 'mistral', 'ortools'
]
TIMEOUT = 1800.0
infos_file = '/public/xcsp_instances/new/scripts/solvers/cop/infos'
portfolios_file = '/public/xcsp_instances/new/scripts/solvers/cop/portfolios_cop'

def compute_stats(solver, infos):
  asp = 0.0
  pof = 0.0
  psi = 0.0
  aot = 0.0
  ast = 0.0
  counter = 0
  for (inst, item) in sorted(infos.items()):
    counter += 1
    score = item[solver]['score']
    time  = item[solver]['time']
    asp += score
    if score > 0:
      psi += 1
      if score == 1:
	aot += time
	pof += 1
      else:
	aot += TIMEOUT
      ast += time
    else:
      ast += TIMEOUT
      aot += TIMEOUT
  asp = str(100 * float(asp) / counter)
  pof = str(100 * float(pof) / counter)
  psi = str(100 * float(psi) / counter)
  aot = str(aot / counter)
  ast = str(ast / counter)
  return solver + '|' + asp + '|' + pof + '|' + psi + '|' + aot + '|' + ast

def compute_stats_vbs(portfolio, infos):
  asp = 0.0
  pof = 0.0
  psi = 0.0
  aot = 0.0
  ast = 0.0
  counter = 0
  for (inst, item) in sorted(infos.items()):
    counter += 1
    port_infos = [
      (1 - float(item[s]['score']), 
           float(item[s]['time']), 
           s) 
      for s in portfolio
    ]
    rank = sorted(port_infos) 
    (score, time, s) = rank[0]
    score = 1 - score
    asp += score
    if score > 0:
      psi += 1
      if score == 1:
	aot += time
	pof += 1
      else:
	aot += TIMEOUT
      ast += time
    else:
      ast += TIMEOUT
      aot += TIMEOUT
  asp = str(100 * float(asp) / counter)
  pof = str(100 * float(pof) / counter)
  psi = str(100 * float(psi) / counter)
  aot = str(aot / counter)
  ast = str(ast / counter)
  return 'vbs_' + str(len(portfolio)) + '|' + asp + '|' + pof + '|' + psi + '|' + aot + '|' + ast
     
def main(args):
  with open(infos_file, 'r') as infile:
    infos = json.load(infile)
  print 'solver|asp|pof|psi|aot|ast'
  for solver in solvers:
    cp = compute_stats(solver, infos)
    print cp
  portfolios_reader = csv.reader(open(portfolios_file,  'rb'),  delimiter = '|')
  for row in portfolios_reader:
    cp = compute_stats_vbs(eval(row[1]), infos)
    print cp

if __name__ == '__main__':
    main(sys.argv[1:])

