'''
Fix inconsistent outcomes.
'''
import sys
import csv
import json

info_file = '/public/xcsp_instances/new/scripts/solvers/cop/infos'
solvers_rank = []
num_solvers = 12
results = {}
feat_vector = {}
infos = {}
voters = 0
abstained = 0

def init_results():
  global results, voters, abstained
  voters = 0
  abstained = 0
  results = {
    'bprolog': 0, 'fzn2smt': 0, 'g12cpx': 0, 'g12fd': 0, 'g12lazyfd': 0, 
    'g12mip': 0, 'gecode': 0, 'izplus': 0, 'jacop': 0, 'minisatid': 0, 
    'mistral': 0, 'ortools': 0
  }
  
def compute_score(info, min_val, max_val, obj, val):
  if info == "unknown":
    return 0
  if info != "sat":
    return 1
  if min_val == max_val:
    return 0.75
  s = ((val - min_val) * 0.5) / (max_val - min_val)
  assert(0 <= s <= 0.5)
  if obj == "min":
    return 0.75 - s
  if obj == "max":
    return s + 0.25
  print "Error!"
  system.exit(1)

def approval_rule(inst):
  global voters, abstained
  solvers = infos[inst]
  approval_list = filter(lambda item: item[1]['info'] != 'unknown', solvers.items())
  if not approval_list:  
    abstained += 1
    return
  voters += 1
  for (solv, _) in approval_list:
    results[solv] += solvers[solv]['score']
      
def borda_rule(inst):
  global voters, abstained
  solvers = infos[inst]
  approval_list = filter(lambda item: item[1]['info'] != 'unknown', solvers.items())
  if not approval_list:  
    abstained += 1
    return
  voters += 1
  times = []
  for (solv, _) in approval_list:
    score = solvers[solv]['score']
    time  = solvers[solv]['time']
    times.append((1 - score, time, solvers_rank.index(solv), solv))
  rank = sorted(times)
  global num_solvers
  n = num_solvers
  for item in rank:
    results[item[3]] += n
    n -= 1
  
def plurality_rule(inst):
  global voters, abstained
  solvers = infos[inst]
  approval_list = filter(lambda item: item[1]['info'] != 'unknown', solvers.items())
  if not approval_list:  
    abstained += 1
    return
  voters += 1
  times = []
  for (solv, _) in approval_list:
    score = solvers[solv]['score']
    time  = solvers[solv]['time']
    times.append((1 - score, time, solvers_rank.index(solv), solv))
  best = min(times)[3]
  global num_solvers
  results[best] += 1

def main(args):
  global infos
  with open(info_file, 'r') as infile:
    infos = json.load(infile)
  init_results()
  for inst in infos.keys():
    approval_rule(inst)
  approval_res = \
    sorted(results.items(), key = lambda item: item[1], reverse = True)
  print "Approval:", approval_res 
  print "Voters:", voters
  print "Abstained:", abstained
  print "Total:", voters + abstained
  print "----------\n"
  global solvers_rank
  solvers_rank = [item[0] for item in approval_res]
  
  init_results()
  for inst in infos.keys():
    borda_rule(inst)
  print "Borda:", \
    sorted(results.items(), key = lambda item: item[1], reverse = True)
  print "Voters:", voters
  print "Abstained:", abstained
  print "Total:", voters + abstained
  print "----------\n"
  
  init_results()
  for inst in infos.keys():
    plurality_rule(inst)
  print "Plurality:", \
    sorted(results.items(), key = lambda item: item[1], reverse = True)
  print "Voters:", voters
  print "Abstained:", abstained
  print "Total:", voters + abstained
  print "----------\n"
  
if __name__ == '__main__':
    main(sys.argv[1:])
